/*
 * lomoco - Logitech Mouse Control for Linux
 *
 * Copyright (c) 2006 by Andreas Schneider <mail@cynapses.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * vim: ts=2 sw=2 et cindent
 */

#include <stdio.h>
#include <stdlib.h>

#include "lomoco_logitech.h"

#include "lomoco_debug.h"
#include "lomoco_list.h"

lomoco_device_t *lomoco_get_logitech_usb_device(lomoco_base_t *base, int pid) {
  lomoco_list_t *logitech_device_list;
  logitech_device_t *logitech_device;

  lomoco_device_t *lomoco_device = NULL;

  /* check for coreded mice */
  for (logitech_device_list = lomoco_list_first(base->logitech_mice_corded); logitech_device_list != NULL;
    logitech_device_list = lomoco_list_next(logitech_device_list)) {

    logitech_device = (logitech_device_t *) logitech_device_list->data;

    /* Do we have a supported device attached */
    if ((pid == logitech_device->pid) && (! logitech_device->hiddev)) {
      LOMOCO_DEBUG(7, "Logitech %s found, vid: 0x%04x, pid: 0x%04x",
                  logitech_device->name, base->logitech_vid,
                  logitech_device->pid);

      lomoco_device = calloc(1, sizeof(lomoco_device_t));

      lomoco_device->vid = base->logitech_vid;
      lomoco_device->pid = logitech_device->pid;
      lomoco_device->name = logitech_device->name;

      goto done;
    }
  }

  /* check for usb receivers */
  for (logitech_device_list = lomoco_list_first(base->logitech_receiver); logitech_device_list != NULL;
    logitech_device_list = lomoco_list_next(logitech_device_list)) {

    logitech_device = (logitech_device_t *) logitech_device_list->data;

    /* Do we have a supported device attached */
    if ((pid == logitech_device->pid) && (! logitech_device->hiddev)) {
      LOMOCO_DEBUG(7, "Logitech %s found, vid: 0x%04x, pid: 0x%04x",
                  logitech_device->name, base->logitech_vid,
                  logitech_device->pid);

      /* TODO check the usb receiver for connected mice */

      lomoco_device = calloc(1, sizeof(lomoco_device_t));

      lomoco_device->vid = base->logitech_vid;
      lomoco_device->pid = logitech_device->pid;
      lomoco_device->name = logitech_device->name;

      goto done;
    }
  }

done:
  return lomoco_device;
}

lomoco_device_t *lomoco_get_logitech_hid_device(lomoco_base_t *base, int pid) {
  lomoco_list_t *logitech_device_list;
  logitech_device_t *logitech_device;

  lomoco_device_t *lomoco_device = NULL;

  /* check for coreded mice */
  for (logitech_device_list = lomoco_list_first(base->logitech_mice_corded); logitech_device_list != NULL;
    logitech_device_list = logitech_device_list->next) {

    logitech_device = (logitech_device_t *) logitech_device_list->data;

    /* Do we have a supported device attached */
    if ((pid == logitech_device->pid) && (logitech_device->hiddev)) {
      LOMOCO_DEBUG(7, "Logitech %s found, vid: 0x%04x, pid: 0x%04x",
                  logitech_device->name, base->logitech_vid,
                  logitech_device->pid);

      lomoco_device = calloc(1, sizeof(lomoco_device_t));

      lomoco_device->vid = base->logitech_vid;
      lomoco_device->pid = logitech_device->pid;
      lomoco_device->name = logitech_device->name;

      goto done;
    }
  }

  /* TODO cordless mice */
  for (logitech_device_list = lomoco_list_first(base->logitech_mice_cordless); logitech_device_list != NULL;
    logitech_device_list = logitech_device_list->next) {

    logitech_device = (logitech_device_t *) logitech_device_list->data;

    /* Do we have a supported device attached */
    if ((pid == logitech_device->pid) && (logitech_device->hiddev)) {
      LOMOCO_DEBUG(7, "Logitech %s found, vid: 0x%04x, pid: 0x%04x",
                  logitech_device->name, base->logitech_vid,
                  logitech_device->pid);

      lomoco_device = calloc(1, sizeof(lomoco_device_t));

      lomoco_device->vid = base->logitech_vid;
      lomoco_device->pid = logitech_device->pid;
      lomoco_device->name = logitech_device->name;

      goto done;
    }
  }


done:
  return lomoco_device;
}
