/*
 * logitech - Logitech Mouse Control for Linux
 *
 * Copyright (c) 2007 by Andreas Schneider <mail@cynapses.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * vim: ts=2 sw=2 et cindent
 */

#ifndef _LOMOCO_LOGITECH_H
#define _LOMOCO_LOGITECH_H

#include "lomoco.h"

enum {
  LOGITECH_CORDED_MOUSE,
#define LOGITECH_CORDED_MOUSE LOGITECH_CORDED_MOUSE
  LOGITECH_CORDLESS_MOUSE,
#define LOGITECH_CORDLESS_MOUSE LOGITECH_CORDLESS_MOUSE
  LOGITECH_RECEIVER
};

/**
 * @struct logitech_mouse_s
 *
 * Used to store the information about the features of a cordless mouse.
 */

/**
 * @typedef logitech_mouse_t
 * Creates a type name for logitech_mouse_s
 */
typedef struct logitech_device_s {
  /** Name of the mouse */
  char name[128];
  /** Product ID */
  int pid;
  /** Device type  */
  int type;
  /** Receiver id of the mouse */
  int rid;
  /** Receiver id of the mouse on 2nd channel */
  int rid2;
  /** Maximum resolution */
  int max_res;
  /** Minimum resolution */
  int min_res;
  /** Resolution step */
  int step;
  /** Smart control reporting */
  int ssr;
  /** Smart control support */
  int sms;
  /** Free Spin control */
  int fsp;
  /** Click-to-Click control */
  int ctc;
  /** Has a second hiddev */
  int hiddev;
  /** Dual receiver */
  int dual;
} logitech_device_t;

/**
 * @brief Check if the the attached Logitech device is supported.
 *
 * @param base The initialized base structure.
 * @param pid The product id of the attached device we are looking for.
 * 
 * @return The supported device.
 */
lomoco_device_t *lomoco_get_logitech_usb_device(lomoco_base_t *base, int pid);

/**
 * @brief Check if the the attached Logitech hid device is supported.
 *
 * @param base The initialized base structure.
 * @param pid The product id of the attached device we are looking for.
 * 
 * @return The supported device.
 */
lomoco_device_t *lomoco_get_logitech_hid_device(lomoco_base_t *base, int pid);

#endif /* _LOMOCO_LOGITECH_H */

