/*
 * lomoco - Logitech Mouse Control for Linux
 *
 * Copyright (c) 2006 by Andreas Schneider <mail@cynapses.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * vim: ts=2 sw=2 et cindent
 */

#include <errno.h>
#include <stdlib.h>

#include "lomoco_config.h"
#include "lomoco_debug.h"
#include "lomoco_list.h"
#include "lomoco_scan.h"

#include "logitech/lomoco_logitech.h"

#include "lomoco.h"

/* initialize lomoco */
lomoco_base_t *lomoco_init(int debug_level) {
  lomoco_base_t *base;
  lomoco_set_debug_level(debug_level);

  /* Allocate the base struct */
  base = calloc(1, sizeof(lomoco_base_t));
  if (base == NULL) {
    LOMOCO_ERRNO(errno, "Failed allocating base struct:");
  }
  base->logitech_mice_corded = NULL;
  base->logitech_mice_cordless = NULL;
  base->logitech_receiver = NULL;

  /* Read the config */
  base = lomoco_load_config(base);

  return base;
}

int lomoco_get_devices(lomoco_base_t *base, lomoco_device_func fn) {
  lomoco_list_t *list = NULL;

  /* TODO error handling */
  list = lomoco_get_usb_devices(base, list);

  list = lomoco_get_hid_devices(base, list);

  if (list == NULL) {
    return 0;
  }

  for (list = lomoco_list_first(list); list != NULL; list = lomoco_list_next(list)) {
    fn((lomoco_device_t *) list->data);

    free(list->data);
  }

  lomoco_list_free(list);

  return 1;
}

void lomoco_finish(lomoco_base_t *base) {
  if (base == NULL) {
    return;
  }
  if (base->logitech_mice_corded) {
    lomoco_list_free(base->logitech_mice_corded);
  }
  if (base->logitech_mice_cordless) {
    lomoco_list_free(base->logitech_mice_cordless);
  }
  if (base->logitech_receiver) {
    lomoco_list_free(base->logitech_receiver);
  }

  free(base);
  base = NULL;
}

