/*
 * lomoco - Logitech Mouse Control for Linux
 *
 * Copyright (c) 2006 by Andreas Schneider <mail@cynapses.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  021100301, USA.
 *
 * vim: ts=2 sw=2 et cindent
 */

#include <iniparser.h>
#include <limits.h>
#include <string.h>

#include "lomoco_config.h"
#include "lomoco_debug.h"
#include "lomoco_list.h"

#include "logitech/lomoco_logitech.h"

#include "config.h"

lomoco_base_t *lomoco_load_config(lomoco_base_t *base) {
  logitech_device_t *mouse;
  logitech_device_t *receiver;
  dictionary *dict = NULL;
  int i;

  char config_file[PATH_MAX];
  char config_value[256];
  char *sec_name;

#if 0
  /* TODO FIXME */
  /* build the path to the config file */
  snprintf(config_file, (size_t) PATH_MAX - 1, "%s/lomoco.ini", SYSCONFDIR);
#endif
  snprintf(config_file, (size_t) PATH_MAX - 1, "./lomoco.ini");
  LOMOCO_DEBUG(1, "Configuration file: %s", config_file);

  /* Load ini file */
  dict = iniparser_load(config_file);
  if (dict == NULL) {
    LOMOCO_ERROR("Could not load config file %s", config_file);
    /* TODO error handling */
    base->err_mask = 0;
    return base;
  }

  /* walk over the ini file */
  for (i = 0; i < iniparser_getnsec(dict); i++) {
    sec_name = iniparser_getsecname(dict, i);
    LOMOCO_DEBUG(8, "Processing section %s", sec_name);

    /* Add corded Logitech mice */
    if (strncmp(sec_name, "logitech/mouse/corded/", (size_t) 22) == 0) {
      mouse = calloc(1, sizeof(logitech_device_t));

      snprintf(config_value, sizeof(config_value) - 1, "%s:name", sec_name);
      strncpy(mouse->name, iniparser_getstring(dict, config_value, NULL),
	      sizeof(mouse->name) - 1);

      snprintf(config_value, sizeof(config_value) - 1, "%s:pid", sec_name);
      mouse->pid = iniparser_getint(dict, config_value, 0);

      mouse->type = LOGITECH_CORDED_MOUSE;

      snprintf(config_value, sizeof(config_value) - 1, "%s:max_res", sec_name);
      mouse->max_res = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:min_res", sec_name);
      mouse->min_res = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:step", sec_name);
      mouse->step = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:ssr", sec_name);
      mouse->ssr = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:sms", sec_name);
      mouse->sms = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:fsp", sec_name);
      mouse->fsp = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:ctc", sec_name);
      mouse->ctc = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:hiddev", sec_name);
      mouse->hiddev = iniparser_getint(dict, config_value, 0);

      base->logitech_mice_corded = lomoco_list_append(base->logitech_mice_corded, mouse);
      LOMOCO_DEBUG(9, "Appended \"%s\" to logitech_mice_corded", mouse->name, mouse->pid);

      continue;
    }

    /* Add cordless Logitech mice */
    if (strncmp(sec_name, "logitech/mouse/cordless/", (size_t) 24) == 0) {
      mouse = calloc(1, sizeof(logitech_device_t));

      snprintf(config_value, sizeof(config_value) - 1, "%s:name", sec_name);
      strncpy(mouse->name, iniparser_getstring(dict, config_value, NULL),
	      sizeof(mouse->name) - 1);

      snprintf(config_value, sizeof(config_value) - 1, "%s:pid", sec_name);
      mouse->pid = iniparser_getint(dict, config_value, 0);

      mouse->type = LOGITECH_CORDLESS_MOUSE;

      snprintf(config_value, sizeof(config_value) - 1, "%s:type", sec_name);
      mouse->rid = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:rid2", sec_name);
      mouse->rid2 = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:max_res", sec_name);
      mouse->max_res = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:min_res", sec_name);
      mouse->min_res = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:step", sec_name);
      mouse->step = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:ssr", sec_name);
      mouse->ssr = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:sms", sec_name);
      mouse->sms = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:fsp", sec_name);
      mouse->fsp = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:ctc", sec_name);
      mouse->ctc = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:hiddev", sec_name);
      mouse->hiddev = iniparser_getint(dict, config_value, 0);

      base->logitech_mice_cordless = lomoco_list_append(base->logitech_mice_cordless, mouse);
      LOMOCO_DEBUG(9, "Appended \"%s\" to logitech_mice_cordless", mouse->name, mouse->pid);

      continue;
    }

    /* Add Logitech receivers */
    if (strncmp(sec_name, "logitech/receiver/", (size_t) 18) == 0) {
      receiver = calloc(1, sizeof(logitech_device_t));

      snprintf(config_value, sizeof(config_value) - 1, "%s:name", sec_name);
      strncpy(receiver->name, iniparser_getstring(dict, config_value, NULL),
	      sizeof(mouse->name) - 1);

      receiver->type = LOGITECH_RECEIVER;

      snprintf(config_value, sizeof(config_value) - 1, "%s:pid", sec_name);
      receiver->pid = iniparser_getint(dict, config_value, 0);

      snprintf(config_value, sizeof(config_value) - 1, "%s:dual", sec_name);
      receiver->dual = iniparser_getint(dict, config_value, 0);

      base->logitech_receiver = lomoco_list_append(base->logitech_receiver, receiver);
      LOMOCO_DEBUG(9, "Appended \"%s\" to logitech_receiver", receiver->name, receiver->pid);

      continue;
    }

    /* Add Logitech vendor id */
    if (strcmp(sec_name, "logitech") == 0) {
      base->logitech_vid = iniparser_getint(dict, "logitech:vid", 0);
      LOMOCO_DEBUG(9, "Vendor ID: 0x%04x", base->logitech_vid);

      continue;
    }
  }

  iniparser_freedict(dict);

  base->err_mask = 0;
  return base;
}

