/*
 * lomoco - Logitech Mouse Control for Linux
 *
 * debug.h form XQF - Quake server browser and launcher
 * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
 * Copyright (c) 2006 by Andreas Schneider <mail@cynapses.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * vim: ts=2 sw=2 et cindent
 */

#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#include "lomoco_debug.h"

static int lomoco_indent_level;
static int lomoco_debug_level;

void lomoco_debug_int(const char* file, int line, const char* function, int level, int err, const char* fmt, ...) {
  va_list argp;
  int i;
  char buf[9];
  char err_buf[256] = {0};
  time_t now;

  for (i = 0; i < lomoco_indent_level; i++) {
    fprintf(stderr, " ");
  }

  now = time(NULL);
  strftime(buf, 9, "%T", localtime(&now));

  if (err > 0) {
    strerror_r(err, err_buf, sizeof(buf) - strlen(buf) - 1);
  }

  switch(level) {
  case -4:
    fprintf(stderr, "ERRNO! %s %s:%d %s() - %s", buf, file, line, function, err_buf);
    break;
  case -3:
    fprintf(stderr, "ERROR! %s %s:%d %s() - ", buf, file, line, function);
    break;
  case -2:
    fprintf(stderr, "WARNING %s %s:%d %s() - ", buf, file, line, function);
    break;
  case -1:
    fprintf(stderr, "NOTE %s %s:%d %s() - ", buf, file, line, function);
    break;
  default:
    fprintf(stderr, "DEBUG(%d) %s %s:%d %s() - ", level, buf, file, line, function);
  }

  va_start(argp, fmt);
  vfprintf(stderr, fmt, argp);
  va_end(argp);
  fprintf(stderr, "\n");
}

void lomoco_debug_cmd(int level, char *argv[], char *fmt, ...) {
  va_list argp;
  int i;
  if (level > lomoco_debug_level) {
    return;
  }

  fprintf(stderr, "DEBUG(%d): ", level);
  va_start(argp, fmt);
  vfprintf(stderr, fmt, argp);
  va_end(argp);
  fprintf(stderr, "  EXEC> ");

  for (i = 0; argv[i]; ++i) {
    fprintf (stderr, "%s ", argv[i]);
  }

  fprintf(stderr, "\n");
}


void lomoco_set_debug_level (int level) {
  lomoco_debug_level = level;
}

inline int lomoco_get_debug_level (void) {
  return (lomoco_debug_level);
}

int lomoco_debug_increase_indent() {
  lomoco_indent_level++;
  return lomoco_indent_level;
}

  int lomoco_debug_decrease_indent() {
  if (lomoco_indent_level > 0) {
    lomoco_indent_level--;
  }
  return lomoco_indent_level;
}

